/***************************************************************************
                           level.c
    description          : level handling
                           -------------------
    begin                : Fri March 30 2001, from rungame.c
    date                 : 01/08/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <limits.h>     //UINT_MAX
#include "aliens.h"     //aliens_set_{count,speed}()
#include "bombs.h"      //bombs_set_{count,speed}()
#include "data.h"       //LoadImage()
#include "hi_score.h"   //digit_30_image
#include "shots.h"      //shots_set_timer()
#include "structs.h"    //screen
#include "level.h"


static SDL_Surface *level_label = NULL;

typedef struct _s_vars {
	int count;
	int speed;
} SVars;

typedef struct _level_s {
	unsigned max;               //max score for this level
	SVars aliens[NUM_ALIENS];   //NUM_ALIENS is currently 3
	SVars bombs;
	int s_time;                 //shot timer.
} Level_s;

#define NUM_LEVELS 5
/* to add level, increment the above NUM_LEVELS define and
   add the variable list below, use UINT_MAX for last level.max*/

//    max        slow     fast      mega      bombs    shot_timer
static Level_s levels[NUM_LEVELS] = {
	{ 500,     {{10, 2}, {0,  0},  {0, 0}},  {5,  2},  50 }, //level 1
	{ 2000,    {{10, 2}, {2,  3},  {1, 1}},  {8,  4},  45 }, //level 2
	{ 4000,    {{10, 4}, {4,  6},  {2, 2}},  {15, 5},  40 }, //level 3
	{ 10000,   {{15, 6}, {8,  8},  {5, 3}},  {20, 6},  35 }, //level 4
	{ UINT_MAX,{{25, 8}, {12, 10}, {10, 5}}, {30, 8},  25 }, //level 5
};


/**
 *  level_init()
 *  initializes level data
 *
 *  Returns:    -1 on error, otherwise 1.
 *
**/
int level_init(void)
{
	if ((level_label = LoadImage(DATAFILE("level.gif"), 0)) == NULL)
		return -1;
	return 1;
}


/**
 *  level_free()
 *  frees level data
 *
**/
void level_free(void)
{
	if (level_label)
		SDL_FreeSurface(level_label);
}


/**
 *  level_check()
 *  check if score >= level.max
 *
 *  @level:     which level.
 *  @score:     test score.
 *
 *  Returns:    1 if score is >= level.max, else 0.
**/
int level_check(int level, int score)
{
	if (score >= levels[level - 1].max || level == 0)
		return 1;
	return 0;
}


#define LE_X            240     // x offset of "level"
#define LE_Y            220     // y offset
#define LE_DIGIT_X      370     // x offset of the (level) digit
#define LE_DIGIT_W      30      // width of digit
#define LE_H            30
#define LE_W            (LE_DIGIT_X - LE_X) + LE_DIGIT_W

/**
 *  level_set_level()
 *  draw level screen and set level data
 *
 *  @level:     which level, clamped 1 to NUM_LEVELS
 *
**/
void level_set_level(int level)
{
	static SDL_Rect bg_rect = {LE_X, LE_Y, LE_W, LE_H};
	SDL_Rect srect, drect;
	int i;

	//Sanity check
	if (level < 1)
		level = 1;
	if (level > NUM_LEVELS)
		level =  NUM_LEVELS;

	//Clear Screen
	srect.x = drect.x = 0;
	srect.y = drect.y = HU_Y + HU_H; //leave heads-up
	srect.w = screen->w;
	srect.h = screen->h - srect.x;
	SDL_BlitSurface(background, &srect, screen, &drect);
	SDL_UpdateRect(screen, drect.x, drect.y, drect.w, drect.h);

	//Draw label
	srect.x = 0;
	srect.y = 0;
	drect.x = LE_X;
	drect.y = LE_Y;
	srect.w = drect.w = level_label->w;
	srect.h = drect.h = level_label->h;
	SDL_LowerBlit(level_label, &srect, screen, &drect);

	//Draw level digit
	if ( level > 0 && level <= NUM_LEVELS )
	{
		srect.x = LE_DIGIT_W * level;
		srect.y = 0;
		drect.x = LE_DIGIT_X;
		drect.y = LE_Y;
		srect.w = drect.w = LE_DIGIT_W;
		srect.h = drect.h = LE_H;
		SDL_LowerBlit(digit_30_image, &srect, screen, &drect);
	}
	SDL_UpdateRect(screen, LE_X, LE_Y, LE_W, LE_H);

	//Set the level vars
	Level_s *l = &levels[level - 1];
	for (i = SLOW_ALIEN; i < NUM_ALIENS; ++i)
	{
		aliens_set_count(i, l->aliens[i].count);
		aliens_set_speed(i, l->aliens[i].speed);
	}
	bombs_set_count(l->bombs.count);
	bombs_set_speed(l->bombs.speed);
	shots_set_timer(l->s_time);

	SDL_Delay(3000);  //Hang around for a bit

	//Erase
	SDL_LowerBlit(background, &bg_rect, screen, &bg_rect);
	SDL_UpdateRect(screen, LE_X, LE_Y, LE_W, LE_H);
}
