/***************************************************************************
                          rungame.c
  description: the main game loop and functions
                             -------------------
    begin                : Fri March 30 2001
    date                 : 27/07/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "structs.h" //global screen, background
#include "aliens.h"
#include "bombs.h"
#include "data.h"
#include "hi_score.h" // big_images
#include "level.h"
#include "player.h"
#include "shots.h"
#include "sound.h"
#include "sprite.h"
#include "rungame.h"


#define JOY_AXIS        0
#define JOY_FIRE        0
#define KEY_FIRE        SDLK_SPACE


typedef enum  {
	GE_NONE,
	GE_LEFT  = 1,
	GE_RIGHT = 1 << 1,
	GE_FIRE  = 1 << 2,
	GE_QUIT  = 1 << 3
} GameAction;


//The in-game score counter
static SDL_Surface *score_image;
static Sprite score_label;

//The lives display
static SDL_Surface *lives_image; //The heart
static Sprite lives_label;


/**
 * rungame_init()
 * init the images, sprites etc.
 *
 * Returns:     -1 on error, otherwise count of Sprites.
 *
**/
int rungame_init(void)
{
	int i;
	int tally = 0;

	if ((score_image = LoadImage(DATAFILE("digit_10_white.gif"), 1)) == NULL)
		return -1;
	if ((score_label.image = LoadImage(DATAFILE("score.gif"), 1)) == NULL)
		return -1;
	score_label.x = 5;
	score_label.y = 5;
	tally++;

	if ((lives_image = LoadImage(DATAFILE("life.gif"), 0)) == NULL)
		return -1;
	if ((lives_label.image = LoadImage(DATAFILE("lives.gif"), 1)) == NULL)
		return -1;
	lives_label.x = 220;
	lives_label.y = 5;
	tally++;

	if ((i = player_init()) < 0)
		return -1;
	tally += i;

	if ((i = aliens_init()) < 0)
		return -1;
	tally += i;

	if ((i = bombs_init()) < 0)
		return -1;
	tally += i;

	if ((i = shots_init()) < 0)
		return -1;
	tally += i;

	if (level_init() < 0)
		return -1;

	return tally;
}
/*******************************************************************/


void rungame_free(void)
{
	level_free();
	shots_free();
	player_free();
	aliens_free();
	SDL_FreeSurface(score_label.image);
	SDL_FreeSurface(score_image);
	SDL_FreeSurface(lives_label.image);
	SDL_FreeSurface(lives_image);
}
/*******************************************************************/


//Do all sprites.
static void _rungame_do_sprites(SpriteFunc dofunc)
{
	player_do(dofunc);
	aliens_foreach(dofunc);
	shots_foreach(dofunc);
	bombs_foreach(dofunc);
}
/*******************************************************************/


static void _rungame_show_lives(int lives)
{
	static SDL_Rect bg_rect = {HU_LIVES_X, HU_Y, MAX_LIVES * HU_FONT_W, HU_H};
	static SDL_Rect drect = {HU_LIVES_X, HU_Y, 10, 10};
	static SDL_Rect srect = {0, 0, 10, 10};
	int i;

	if (lives > MAX_LIVES || lives < 0) // sanity check
		lives = 0;

	//Erase
	SDL_LowerBlit(background, &bg_rect, screen, &bg_rect);

	//Draw
	drect.x = HU_LIVES_X;
	for (i = 0; i < lives; ++i)
	{
		SDL_LowerBlit(lives_image, &srect, screen, &drect);
		drect.x += HU_FONT_W;
	}
	SDL_UpdateRect(screen, HU_LIVES_X, HU_Y, MAX_LIVES * HU_FONT_W, 10);
}
/********************************************************************/


static void _rungame_show_score(int number)
{
	static SDL_Rect bg_rect = {HU_SCORE_X, HU_Y, MAX_DIGITS * HU_FONT_W, HU_H};
	static SDL_Rect srect = {0, 0, 10, HU_H};
	static SDL_Rect drect = {HU_SCORE_X, HU_Y, 10, HU_H};
	char score_int[MAX_DIGITS + 2];
	int i, digits;

	if (number > SCORE_MAX || number < 0)	// sanity check
		number = SCORE_MAX;
	digits = snprintf(score_int, MAX_DIGITS + 1, "%d", number);

	//Erase
	SDL_LowerBlit(background, &bg_rect, screen, &bg_rect);

	//Draw
	drect.x = HU_SCORE_X;
	for (i = 0; i < digits; ++i)
	{
		srect.x = (score_int[i] - '0') * 10;
		SDL_LowerBlit(score_image, &srect, screen, &drect);
		drect.x += HU_FONT_W;
	}
	SDL_UpdateRect(screen, HU_SCORE_X, HU_Y, MAX_DIGITS * HU_FONT_W, HU_H);
}
/*******************************************************************/


static void _rungame_set_level(int level)
{
	//If player has lost lives in prev level +1 life for reaching next level
	if (level > 1)
	{
		int lives = player_add_life();
		if (lives)
			_rungame_show_lives(lives);
	}
	level_set_level(level);
}
/*******************************************************************/


//The run game function
int RunGame(int *ret_score)
{
	int level = 0;
	int score = 0;
	int ret_state = RUN_FINAL;    //default: run final score after game

	SDL_Event event;
	SDL_Rect dst;

	int action = GE_NONE; //Note: Clear the action before each level

	//Paint the background
	dst.x = 0;
	dst.y = 0;
	dst.w = background->w;
	dst.h = background->h;
	SDL_BlitSurface(background, NULL, screen, &dst);
	SDL_UpdateRect(screen, 0, 0, 0, 0);

	//Kill all, i.e. set all Sprite.alive to 0
	_rungame_do_sprites(sprite_ops[SPRITE_KILL]);

	//Give the player his lives
	player_init_lives();

	//Set numupdates to zero
	numupdates = 0;

	//Create and initialize score for first time '0'
	sprite_ops[SPRITE_DRAW](&score_label);
	_rungame_show_score(0);
	//Create and initialize lives for first time '0'
	sprite_ops[SPRITE_DRAW](&lives_label);
	_rungame_show_lives(player_get_lives());
	UpdateScreen();

	//Game loop
	while (player_get_lives())
	{
		int points = 0; //Reset points for this frame
		int death = 0;  //Reset death for this frame

	//Wait for the next frame
		WaitFrame();

	//New Level?
		if (level_check(level, score))
		{
			++level;
			action = GE_NONE;
			_rungame_set_level(level);
		}

	//Get input
		while ( SDL_PollEvent(&event) )
		{
			switch (event.type)
			{
				case SDL_QUIT:
					action = GE_QUIT;
				break;
				case SDL_JOYAXISMOTION:
					if (event.jaxis.axis == JOY_AXIS)
					{
						/* 2% jitter */
						if (event.jaxis.value < -640)
							action |= GE_LEFT;
						else if (event.jaxis.value > 640)
							action |= GE_RIGHT;
						else action &= ~(GE_RIGHT|GE_LEFT);
					}
					break;
				case SDL_JOYBUTTONDOWN:
					if (event.jbutton.button == JOY_FIRE)
						action |= GE_FIRE;
					break;
				case SDL_JOYBUTTONUP:
					if (event.jbutton.button == JOY_FIRE)
						action &= ~GE_FIRE;
					break;
				case SDL_KEYDOWN:
					if (event.key.keysym.sym == KEY_FIRE)
						action |= GE_FIRE;
					else if (event.key.keysym.sym == SDLK_LEFT)
						action |= GE_LEFT;
					else if (event.key.keysym.sym == SDLK_RIGHT)
						action |= GE_RIGHT;
					else if (event.key.keysym.sym == SDLK_ESCAPE)
						action = GE_QUIT;
					break;
				case SDL_KEYUP:
					if (event.key.keysym.sym == KEY_FIRE)
						action &= ~GE_FIRE;
					else if (event.key.keysym.sym == SDLK_LEFT)
						action &= ~GE_LEFT;
					else if (event.key.keysym.sym == SDLK_RIGHT)
						action &= ~GE_RIGHT;
					break;
				default:
					break;
			}
		}

	//User wants to quit?
		if (action == GE_QUIT)
		{
			ret_state = RUN_QUIT;
			goto rungame_exit;
		}

	//Erase everthing
		_rungame_do_sprites(sprite_ops[SPRITE_ERASE]);

	//Move everything
		aliens_move();
		bombs_move();
		shots_move();
		if (action & GE_RIGHT)
			player_right();
		else if (action & GE_LEFT)
			player_left();

	//Create new aliens, bombs and shots
		aliens_add();
		aliens_add_bomb();
		if (action & GE_FIRE)
			player_add_shot();

	//Detect collisions
		death = player_collide();
		points = shots_collide();
		//Collisons?
		if (points || death)
		{
			//Sound Explosion
			Mix_PlayChannel(EXPLODE_WAV,
						sounds[EXPLODE_WAV], 0);
			//Update score?
			if (points)
			{
				score += points;
				if (score >= SCORE_MAX)
				{
					score = SCORE_MAX;
					goto rungame_exit;
				}
				_rungame_show_score(score);
			}
			//Update player and lives?
			if (death)
				_rungame_show_lives(player_get_lives());
		}

	//Redraw Sprites
		_rungame_do_sprites(sprite_ops[SPRITE_DRAW]);
		UpdateScreen();

	//Loop the music
		if (music && !Mix_PlayingMusic())
			Mix_PlayMusic(music, 0);
		else if (sounds[MUSIC_WAV]  && !Mix_Playing(MUSIC_WAV))
			Mix_PlayChannel(MUSIC_WAV, sounds[MUSIC_WAV], 0);
	} //Game loop ends

//Exit
rungame_exit:
	Mix_HaltChannel(-1);
	if (music && Mix_PlayingMusic())
		Mix_HaltMusic();

	*ret_score = score;
	return ret_state;
}
